@@ -1,17 +1,19 @@
 pub use image::{Rgb, Rgba};
 
+#[derive(Debug)]
 pub enum WordBreak {
     Normal,
     BreakAll,
 }
 
-#[derive(Default)]
+#[derive(Debug, Default, Clone)]
 pub struct Margin(pub i32, pub i32, pub i32, pub i32);
 
-#[derive(Default, Clone, Debug)]
+#[derive(Debug, Default, Clone)]
 pub struct BorderRadius(pub u32, pub u32, pub u32, pub u32);
 
 /// Adjust the horizontal position.
+#[derive(Debug)]
 pub enum AlignItems {
     Start,
     Center,
@@ -19,6 +21,7 @@ pub enum AlignItems {
 }
 
 /// Adjust the vertical position.
+#[derive(Debug)]
 pub enum JustifyContent {
     Start,
     Center,
@@ -26,25 +29,35 @@ pub enum JustifyContent {
 }
 
 /// Adjust the text horizontal position.
+#[derive(Debug)]
 pub enum TextAlign {
     Start,
     Center,
     End,
 }
 
+#[derive(Debug)]
 pub enum TextOverflow<'a> {
     Clip,
     Ellipsis,
     Content(&'a str),
 }
 
+#[derive(Debug)]
 pub enum Position {
     Static,
     Absolute,
 }
 
+#[derive(Debug)]
+pub enum FlexDirection {
+    Column,
+    Row,
+}
+
 /// Style is used by `text` or `img` element.
 /// Text element is `inline-block`, so you can adjust text position by using `text_align`.
+#[derive(Debug)]
 pub struct Style<'a> {
     pub margin: Margin,
     /// For Text element
@@ -52,15 +65,16 @@ pub struct Style<'a> {
     /// For Text element
     pub font_size: f32,
     /// For Text element
-    pub font_family: Option<Vec<u8>>,
-    /// For Text element
     pub word_break: WordBreak,
     /// For Text element
     pub color: Rgba<u8>,
+    /// For Text element
     pub text_align: TextAlign,
     /// For Text element
     pub max_height: Option<u32>,
     /// For Text element
+    pub max_width: Option<u32>,
+    /// For Text element
     /// This property support multiline.
     pub text_overflow: TextOverflow<'a>,
     pub position: Position,
@@ -78,11 +92,11 @@ impl<'a> Default for Style<'a> {
             margin: Margin::default(),
             line_height: 1.5,
             font_size: 30.,
-            font_family: None,
             word_break: WordBreak::Normal,
             color: Rgba([0, 0, 0, 255]),
             text_align: TextAlign::Start,
             max_height: None,
+            max_width: None,
             text_overflow: TextOverflow::Clip,
             position: Position::Static,
             top: None,
@@ -94,15 +108,41 @@ impl<'a> Default for Style<'a> {
     }
 }
 
+pub enum LogicalFlexRowPosition {
+    Start,
+    Center,
+    End,
+}
+
 /// Window is act like flexbox. And default direction is `column`.
 /// You can adjust position with `align_item` and `justify_content`.
 /// You must pass `background_image` or `background_color` for constructing surface.
+#[derive(Debug)]
 pub struct WindowStyle {
     pub height: u32,
     pub width: u32,
     pub background_color: Option<Rgba<u8>>,
     pub align_items: AlignItems,
     pub justify_content: JustifyContent,
+    /// This controls the direction in which the children of a node are laid out.
+    pub flex_direction: FlexDirection,
+}
+
+impl WindowStyle {
+    pub fn logical_flex_row_position(&self) -> LogicalFlexRowPosition {
+        match &self.flex_direction {
+            FlexDirection::Column => match self.align_items {
+                AlignItems::Start => LogicalFlexRowPosition::Start,
+                AlignItems::Center => LogicalFlexRowPosition::Center,
+                AlignItems::End => LogicalFlexRowPosition::End,
+            },
+            FlexDirection::Row => match self.justify_content {
+                JustifyContent::Start => LogicalFlexRowPosition::Start,
+                JustifyContent::Center => LogicalFlexRowPosition::Center,
+                JustifyContent::End => LogicalFlexRowPosition::End,
+            },
+        }
+    }
 }
 
 impl Default for WindowStyle {
@@ -113,6 +153,7 @@ impl Default for WindowStyle {
             background_color: None,
             align_items: AlignItems::Start,
             justify_content: JustifyContent::Start,
+            flex_direction: FlexDirection::Column,
         }
     }
 }
